﻿using System.Collections.Generic;
using System.Linq;

using DevExpress.XtraEditors;
using DevExpress.XtraTreeList;
using DevExpress.XtraTreeList.Columns;
using DevExpress.XtraTreeList.Nodes;

namespace TestProject
{
    /// <summary>
    /// 메인 폼
    /// </summary>
    public partial class MainForm : XtraForm
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainForm()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainForm()
        {
            InitializeComponent();

            this.treeList.ShowFindPanel();

            this.treeList.OptionsBehavior.EnableFiltering = true;

            this.treeList.FilterNode += treeList_FilterNode;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 트리 리스트 노드 필터 설정하기 - treeList_FilterNode(sender, e)

        /// <summary>
        /// 트리 리스트 노드 필터 설정하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void treeList_FilterNode(object sender, FilterNodeEventArgs e)
        {
            List<TreeListColumn> columnList = e.Node.TreeList.Columns.Cast<TreeListColumn>().ToList();

            if(columnList.Count == 0)
            {
                return;
            }

            if(string.IsNullOrEmpty(treeList.FindFilterText))
            {
                return;
            }

            e.Handled = true;

            e.Node.Visible = columnList.Any(c => IsNodeMatchFilter(e.Node, c));

            e.Node.Expanded = e.Node.Visible;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region 노드 필터 매칭 여부 구하기 - IsNodeMatchFilter(node, column)

        /// <summary>
        /// 노드 필터 매칭 여부 구하기
        /// </summary>
        /// <param name="node">노드</param>
        /// <param name="column">컬럼</param>
        /// <returns>노드 필터 매칭 여부</returns>
        private bool IsNodeMatchFilter(TreeListNode node, TreeListColumn column)
        {
            string filterValue = this.treeList.FindFilterText.ToLower();

            if(node.GetDisplayText(column).ToLower().Contains(filterValue))
            {
                return true;
            }

            foreach(TreeListNode childNode in node.Nodes)
            {
                if(IsNodeMatchFilter(childNode, column))
                {
                    return true;
                }
            }

            return false;
        }

        #endregion
    }
}